# ES2015

When `vue-loader` detects the presence of `babel-loader` or `buble-loader` in the same project, it will use them to process the `<script>` parts of all `*.vue` files, allowing us to use ES2015 in our Vue components. If you haven't picked up the new language features yet, you definitely should. Some good learning resources:

- [Babel - Learn ES2015](https://babeljs.io/docs/learn-es2015/)
- [ES6 Features](https://github.com/lukehoban/es6features)
- [Exploring ES6 (book)](https://leanpub.com/exploring-es6)

Here's a typical pattern you will see when importing other Vue components:

``` html
<script>
import ComponentA from './ComponentA.vue'
import ComponentB from './ComponentB.vue'

export default {
  components: {
    ComponentA,
    ComponentB
  }
}
</script>
```

We are using ES2015's Object literal shorthand here to define the child components. `{ ComponentA }` is simply shorthand for `{ ComponentA: ComponentA }`. Vue will automatically convert the key to `component-a`, so you can use the imported component in the template as `<component-a>`.

### ES2015 in Templates

Note that expressions inside `<template>`s do not get processed by the same Babel (or Buble) configuration like the `<script>` parts, because some ES2015 features cannot be compiled into render-function-friendly code. However, `vue-loader` does process the generated render function to selectively support a few convenient ES2015 features:

- [Template string interpolation](http://es6-features.org/#StringInterpolation)
- [Object property shorthand](http://es6-features.org/#PropertyShorthand)
- [Object method shorthand](http://es6-features.org/#MethodProperties)
- [Computed property names](http://es6-features.org/#ComputedPropertyNames)

These features are quite convenient in dynamic class bindings, for example:

``` html
<div :class="[{ active: active }, isButton ? prefix + '-button' : null]">
```

can be simplified to:

``` html
<div :class="{ active, [`${prefix}-button`]: isButton }">
```

You can also customize the features supported in tempaltes using the [`templateBuble` option](../options.md#templatebuble).

### Transpiling Normal `.js` Files

Since `vue-loader` only processes `*.vue` files, you'd need to tell Webpack to process normal `*.js` files with `babel-loader` or `buble-loader` in the Webpack config file. The project scaffolded with `vue-cli` already does it for you.

### Configuring Babel with `.babelrc`

`babel-loader` respects [`.babelrc`](https://babeljs.io/docs/usage/babelrc/), so it is the recommended approach to configure the Babel presets and plugins.
